"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vm_1 = require("vm");
const dot_prop_1 = __importDefault(require("dot-prop"));
const Command_1 = __importDefault(require("./Command"));
exports.default = new Command_1.default(async ([key, ...value]) => {
    if (!value.length) {
        const configValue = dot_prop_1.default.get(config.config, key);
        return `\`config.${key}\` is set to \`${typeof configValue === 'string' ? configValue : JSON.stringify(configValue)}\``;
    }
    let val = value.join(' ');
    if (val === 'true')
        val = true;
    if (val === 'false')
        val = false;
    if (/^`.+`$/.test(val)) {
        try {
            val = vm_1.runInNewContext(val.match(/^`(.+)`$/)[1], Object.create(null));
            if (!Array.isArray(val) && !['string', 'number', 'boolean'].includes(typeof val)) {
                return 'Invalid type!';
            }
        }
        catch (err) {
            return err.message;
        }
    }
    await config.set(key, val);
    return `\`config.${key}\` was set to \`${JSON.stringify(val)}\`.`;
}, {
    argc: 1,
    usage: `config <key> [value]`,
    example: `config discord.massMentions true`,
    allowMoreArguments: true
});
